@@ -16,7 +16,7 @@ use Data::MuForm::Localizer;
 use MooX::Aliases;
 use Data::MuForm::Merge ('merge');
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 
 has 'name' => ( is => 'ro', builder => 'build_name');
@@ -115,10 +115,12 @@ sub base_render_args {
     name => $self->name,
     id => $self->name,
     form_errors => $self->form_errors || [],
-    method => $self->http_method,
+    form_attr => {
+      method => $self->http_method,
+    }
   };
-  $args->{action} = $self->action if $self->action;
-  $args->{enctype} = $self->enctype if $self->enctype;
+  $args->{form_attr}->{action} = $self->action if $self->action;
+  $args->{form_attr}->{enctype} = $self->enctype if $self->enctype;
   return $args;
 }
 
@@ -386,10 +388,6 @@ sub validate_form {
     $self->validate;       # hook
     $self->validate_model; # hook
     $self->fields_set_value;
-    # $self->build_errors;
-
-    # 'validated' depends on no errors...
-
     $self->submitted(undef);
     $self->ran_validation(1);
 }
@@ -495,19 +493,21 @@ sub render {
 
 sub render_start {
   my ( $self, $rargs ) = @_;
-  my $render_args = $self->get_render_args(%$rargs, rendering => 'form_start');
+  my $render_args = $self->get_render_args( form_attr => $rargs, rendering => 'form_start');
   return $self->renderer->render_start($render_args);
 }
 
 sub render_end {
   my ( $self, $rargs ) = @_;
-  my $render_args = $self->get_render_args(%$rargs, rendering => 'form_end');
+  # is there any need for attributes on a form end tag? never seen any...
+  my $render_args = $self->get_render_args(rendering => 'form_end');
   return $self->renderer->render_end($render_args);
 }
 
 sub render_errors {
   my ( $self, $rargs ) = @_;
-  my $render_args = $self->get_render_args(%$rargs, rendering => 'form_errors');
+  # we're not doing 'form_error_attr'. only processing 'error_tag' and 'error_class'
+  my $render_args = $self->get_render_args( %$rargs, rendering => 'form_errors');
   return $self->renderer->render_form_errors($render_args);
 }
 
@@ -535,7 +535,7 @@ Data::MuForm - Data validator and form processor
 
 =head1 VERSION
 
-version 0.03
+version 0.04
 
 =head1 SYNOPSIS
 
@@ -899,6 +899,16 @@ method also works:
     $self->field('media')->field('caption'); # works
     $self->subfield('media.caption'); # works
 
+=head3 build_field_id
+
+Create a 'build_field_id' sub in the form class to use a common method for
+constructing field ids.
+
+    sub build_field_id {
+       my ( $self, $field ) = @_;
+       return $field->name . '_' . $self->id;
+    }
+
 =head2 Constraints and validation
 
 Most validation is performed on a per-field basis, and there are a number
