@@ -250,7 +250,7 @@ sub build_input_type { 'text' }
 has 'layout' => ( is => 'rw' );
 has 'layout_group' => ( is => 'rw' );
 has 'order' => ( is => 'rw', default => 0 );
-has 'html5_type_attr' => ( is => 'rw' );
+has 'html5_input_type' => ( is => 'rw', predicate => 'has_html5_input_type' );
 
 sub base_render_args {
   my $self = shift;
@@ -268,6 +268,7 @@ sub base_render_args {
     layout_type => 'standard',
     is_contains => $self->is_contains,
   };
+  $args->{html5_input_type} = $self->html5_input_type if $self->has_html5_input_type;
   return $args;
 }
 
@@ -801,7 +802,7 @@ Data::MuForm::Field - Base field package
 
 =head1 VERSION
 
-version 0.03
+version 0.04
 
 =head1 SYNOPSIS
 
@@ -858,11 +859,16 @@ The only required attribute.
 =item type
 
 The class or type of the field. The 'type' of L<Data::MuForm::Field::Currency>
-is 'Currency'. Classes that you define yourself are prefixed with '+'.
+is 'Currency'.
 
 =item id
 
-The id to use when rendering.
+The id to use when rendering. This can come from a number of different places.
+
+  1) field definition
+  2) field 'build_id' method
+  3) form 'build_field_id' method
+  4) field prefixed_name
 
 =item accessor
 
